<?php























require_once(INCLUDE_PATH."/common.php");

class DB_mssql extends DB_common
{
 var $connection;
 var $phptype, $dbsyntax;
 var $prepare_tokens = array();
 var $prepare_types = array();
 var $transaction_opcount = 0;
 var $autocommit = true;
 var $_db = null;

 function DB_mssql()
 {
 $this->DB_common();
 $this->phptype = 'mssql';
 $this->dbsyntax = 'mssql';
 $this->features = array(
 'prepare' => false,
 'pconnect' => true,
 'transactions' => true,
 'limit' => 'emulate'
 );
 
 $this->errorcode_map = array(

 );
 }

 function connect($dsninfo, $persistent = false)
 {
 if (!DB::assertExtension('mssql') && !DB::assertExtension('sybase')
 && !DB::assertExtension('sybase_ct'))
 {
 return $this->raiseError(DB_ERROR_EXTENSION_NOT_FOUND);
 }
 $this->dsn = $dsninfo;
 $user = $dsninfo['username'];
 $pw = $dsninfo['password'];
 $dbhost = $dsninfo['hostspec'] ? $dsninfo['hostspec'] : 'localhost';

 $connect_function = $persistent ? 'mssql_pconnect' : 'mssql_connect';

 if ($dbhost && $user && $pw) {
 $conn = @$connect_function($dbhost, $user, $pw);
 } elseif ($dbhost && $user) {
 $conn = @$connect_function($dbhost, $user);
 } else {
 $conn = @$connect_function($dbhost);
 }
 if (!$conn) {
 return $this->raiseError(DB_ERROR_CONNECT_FAILED, null, null,
 null, mssql_get_last_message());
 }
 if ($dsninfo['database']) {
 if (!@mssql_select_db($dsninfo['database'], $conn)) {
 return $this->raiseError(DB_ERROR_NODBSELECTED, null, null,
 null, mssql_get_last_message());
 }
 $this->_db = $dsninfo['database'];
 }
 $this->connection = $conn;
 return DB_OK;
 }

 function disconnect()
 {
 $ret = @mssql_close($this->connection);
 $this->connection = null;
 return $ret;
 }

 function simpleQuery($query)
 {
 $ismanip = DB::isManip($query);
 $this->last_query = $query;
 if (!@mssql_select_db($this->_db, $this->connection)) {
 return $this->mssqlRaiseError(DB_ERROR_NODBSELECTED);
 }
 $query = $this->modifyQuery($query);
 if (!$this->autocommit && $ismanip) {
 if ($this->transaction_opcount == 0) {
 $result = @mssql_query('BEGIN TRAN', $this->connection);
 if (!$result) {
 return $this->mssqlRaiseError();
 }
 }
 $this->transaction_opcount++;
 }
 $result = @mssql_query($query, $this->connection);
 if (!$result) {
 return $this->mssqlRaiseError();
 }
 
 
 return $ismanip ? DB_OK : $result;
 }

 

 








 function nextResult($result)
 {
 return mssql_next_result($result);
 }

 

 function &fetchRow($result, $fetchmode = DB_FETCHMODE_DEFAULT, $rownum=null)
 {
 if ($fetchmode == DB_FETCHMODE_DEFAULT) {
 $fetchmode = $this->fetchmode;
 }
 $res = $this->fetchInto ($result, $arr, $fetchmode, $rownum);
 if ($res !== DB_OK) {
 return $res;
 }
 return $arr;
 }

 function fetchInto($result, &$ar, $fetchmode, $rownum=null)
 {
 if ($rownum !== null) {
 if (!@mssql_data_seek($result, $rownum)) {
 return null;
 }
 }
 if ($fetchmode & DB_FETCHMODE_ASSOC) {
 $ar = @mssql_fetch_array($result);
 } else {
 $ar = @mssql_fetch_row($result);
 }
 if (!$ar) {
 





 return null;
 }
 return DB_OK;
 }

 function freeResult($result)
 {
 if (is_resource($result)) {
 return @mssql_free_result($result);
 }
 if (!isset($this->prepare_tokens[$result])) {
 return false;
 }
 unset($this->prepare_tokens[$result]);
 unset($this->prepare_types[$result]);
 return true;
 }

 function numCols($result)
 {
 $cols = @mssql_num_fields($result);
 if (!$cols) {
 return $this->mssqlRaiseError();
 }
 return $cols;
 }

 function numRows($result)
 {
 $rows = @mssql_num_rows($result);
 if ($rows === false) {
 return $this->mssqlRaiseError();
 }
 return $rows;
 }

 


 function autoCommit($onoff = false)
 {
 
 
 $this->autocommit = $onoff ? true : false;
 return DB_OK;
 }

 
 

 


 function commit()
 {
 if ($this->transaction_opcount > 0) {
 if (!@mssql_select_db($this->_db, $this->connection)) {
 return $this->mssqlRaiseError(DB_ERROR_NODBSELECTED);
 }
 $result = @mssql_query('COMMIT TRAN', $this->connection);
 $this->transaction_opcount = 0;
 if (!$result) {
 return $this->mssqlRaiseError();
 }
 }
 return DB_OK;
 }

 
 

 


 function rollback()
 {
 if ($this->transaction_opcount > 0) {
 if (!@mssql_select_db($this->_db, $this->connection)) {
 return $this->mssqlRaiseError(DB_ERROR_NODBSELECTED);
 }
 $result = @mssql_query('ROLLBACK TRAN', $this->connection);
 $this->transaction_opcount = 0;
 if (!$result) {
 return $this->mssqlRaiseError();
 }
 }
 return DB_OK;
 }

 
 

 





 function affectedRows()
 {
 if (DB::isManip($this->last_query)) {
 $res = @mssql_query('select @@rowcount', $this->connection);
 if (!$res) {
 return $this->mssqlRaiseError();
 }
 $ar = @mssql_fetch_row($res);
 if (!$ar) {
 $result = 0;
 } else {
 @mssql_free_result($res);
 $result = $ar[0];
 }
 } else {
 $result = 0;
 }
 return $result;
 }
 

 












 function nextId($seq_name, $ondemand = true)
 {
 $seqname = $this->getSequenceName($seq_name);
 if (!@mssql_select_db($this->_db, $this->connection)) {
 return $this->mssqlRaiseError(DB_ERROR_NODBSELECTED);
 }
 $repeat = 0;
 do {
 $this->pushErrorHandling(PEAR_ERROR_RETURN);
 $result = $this->query("INSERT INTO $seqname (vapor) VALUES (0)");
 $this->popErrorHandling();
 if ($ondemand && DB::isError($result) &&
 ($result->getCode() == DB_ERROR || $result->getCode() == DB_ERROR_NOSUCHTABLE))
 {
 $repeat = 1;
 $result = $this->createSequence($seq_name);
 if (DB::isError($result)) {
 return $result;
 }
 } else {
 $result = $this->query("SELECT @@IDENTITY FROM $seqname");
 $repeat = 0;
 }
 } while ($repeat);
 if (DB::isError($result)) {
 return $this->raiseError($result);
 }
 $result = $result->fetchRow(DB_FETCHMODE_ORDERED);
 return $result[0];
 }

 
 

 function createSequence($seq_name)
 {
 $seqname = $this->getSequenceName($seq_name);
 return $this->query("CREATE TABLE $seqname ".
 '([id] [int] IDENTITY (1, 1) NOT NULL ,' .
 '[vapor] [int] NULL)');
 }
 
 

 function dropSequence($seq_name)
 {
 $seqname = $this->getSequenceName($seq_name);
 return $this->query("DROP TABLE $seqname");
 }
 

 function errorCode()
 {
 $this->pushErrorHandling(PEAR_ERROR_RETURN);
 $error_code = $this->getOne('select @@ERROR as ErrorCode');
 $this->popErrorHandling();
 
 if (!isset($this->errorcode_map[$error_code])) {
 return DB_ERROR;
 }
 return $error_code;
 }

 function mssqlRaiseError($code = null)
 {
 if ($code !== null) {
 $code = $this->errorCode();
 if (DB::isError($code)) {
 return $this->raiseError($code);
 }
 }
 return $this->raiseError($code, null, null, null, mssql_get_last_message());
 }

 












 function tableInfo($result, $mode = null)
 {

 $count = 0;
 $id = 0;
 $res = array();

 




































 
 

 if (is_string($result)) {
 if (!@mssql_select_db($this->_db, $this->connection)) {
 return $this->mssqlRaiseError(DB_ERROR_NODBSELECTED);
 }
 $id = mssql_query("SELECT * FROM $result", $this->connection);
 if (empty($id)) {
 return $this->mssqlRaiseError();
 }
 } else { 
 $id = $result;
 if (empty($id)) {
 return $this->mssqlRaiseError();
 }
 }

 $count = @mssql_num_fields($id);

 
 if (empty($mode)) {

 for ($i=0; $i<$count; $i++) {
 $res[$i]['table'] = (is_string($result)) ? $result : '';
 $res[$i]['name'] = @mssql_field_name($id, $i);
 $res[$i]['type'] = @mssql_field_type($id, $i);
 $res[$i]['len'] = @mssql_field_length($id, $i);
 $res[$i]['flags'] = '';
 }

 } else { 
 $res['num_fields']= $count;

 for ($i=0; $i<$count; $i++) {
 $res[$i]['table'] = (is_string($result)) ? $result : '';
 $res[$i]['name'] = @mssql_field_name($id, $i);
 $res[$i]['type'] = @mssql_field_type($id, $i);
 $res[$i]['len'] = @mssql_field_length($id, $i);
 $res[$i]['flags'] = '';
 if ($mode & DB_TABLEINFO_ORDER) {
 $res['order'][$res[$i]['name']] = $i;
 }
 if ($mode & DB_TABLEINFO_ORDERTABLE) {
 $res['ordertable'][$res[$i]['table']][$res[$i]['name']] = $i;
 }
 }
 }

 
 if (is_string($result)) {
 @mssql_free_result($id);
 }
 return $res;
 }

 

 




 function getSpecialQuery($type)
 {
 switch ($type) {
 case 'tables':
 $sql = "select name from sysobjects where type = 'U' order by name";
 break;
 case 'views':
 $sql = "select name from sysobjects where type = 'V'";
 break;
 default:
 return null;
 }
 return $sql;
 }

 

}
?>
